part of firebase_model_notifier;

extension FirestoreDynamicDocumentModelExtensions
    on FirestoreDynamicDocumentModel {
  String localized(
    String key,
    String orElse, {
    String? locale,
    String localizationValueKey = "@translate",
  }) {
    locale ??= Localize.language;
    final map = get(
      "$key$localeValueKey",
      const <String, dynamic>{},
    );
    if (map.isEmpty) {
      return get(key, orElse);
    }
    return map.get(
      locale,
      get(key, orElse),
    );
  }

  FirestoreDynamicDocumentModel setSearchField({
    String key = "@search",
    List<String> bigramKeys = const ["name", "text"],
    List<String> tagKeys = const ["tag", "category"],
  }) {
    var tmp = "";
    for (final bigramKey in bigramKeys) {
      if (this[bigramKey] is! String) {
        continue;
      }
      tmp += this[bigramKey];
    }
    final res = <String, bool>{};
    final bigramList = tmp.splitByBigram();
    for (final bigram in bigramList) {
      res[bigram] = true;
    }
    for (final tagKey in tagKeys) {
      if (this[tagKey] is! List<String>) {
        continue;
      }
      for (final tag in this[tagKey]) {
        res[tag] = true;
      }
    }
    this[key] = res;
    return this;
  }

  FirestoreDynamicDocumentModel increment(
    String key,
    num value, {
    List<FirestoreCounterUpdaterInterval> intervals = const [],
  }) {
    this[key] = FieldValue.increment(value);
    if (intervals.isEmpty) {
      return this;
    }
    final now = DateTime.now();
    for (final interval in intervals) {
      switch (interval) {
        case FirestoreCounterUpdaterInterval.daily:
          this[dailyKey(key, now)] = FieldValue.increment(value);
          for (var i = 0; i < 30; i++) {
            this[dailyKey(
                    key, DateTime(now.year, now.month, now.day - 60 + i))] =
                FieldValue.delete();
          }
          break;
        case FirestoreCounterUpdaterInterval.monthly:
          this[monthlyKey(key, now)] = FieldValue.increment(value);
          for (var i = 0; i < 12; i++) {
            this[monthlyKey(key, DateTime(now.year, now.month - 24 + i))] =
                FieldValue.delete();
          }
          break;
        case FirestoreCounterUpdaterInterval.yearly:
          this[yearlyKey(key, now)] = FieldValue.increment(value);
          for (var i = 0; i < 5; i++) {
            this[yearlyKey(key, DateTime(now.year, now.month - 10 + i))] =
                FieldValue.delete();
          }
          break;
        case FirestoreCounterUpdaterInterval.weekly:
          this[weeklyKey(key, now)] = FieldValue.increment(value);
          for (var i = 0; i < 4; i++) {
            this[weeklyKey(key,
                    DateTime(now.year, now.month, now.day - ((8 - i) * 7)))] =
                FieldValue.delete();
          }
          break;
      }
    }
    return this;
  }

  FirestoreDynamicDocumentModel deleteField(String key) {
    this[key] = FieldValue.delete();
    return this;
  }

  FirestoreDynamicDocumentModel timestamp(String key) {
    this[key] = FieldValue.serverTimestamp();
    return this;
  }
}
