part of firebase_model_notifier;

abstract class FirebaseFunctionsDocumentModel<T> extends DocumentModel<T> {
  FirebaseFunctionsDocumentModel(this.endpoint, T initialValue)
      : super(initialValue);

  @override
  @protected
  @mustCallSuper
  void initState() {
    super.initState();
    value ??= initialValue;
  }

  @protected
  FirebaseFunctions get functions {
    return FirebaseFunctions.instanceFor(region: FirebaseCore.region);
  }

  @protected
  T get initialValue;

  final String endpoint;

  @protected
  @mustCallSuper
  Future<void> onLoad() async {}

  @protected
  @mustCallSuper
  Future<void> onSave() async {}

  @protected
  @mustCallSuper
  Future<void> onDidLoad() async {}

  @protected
  @mustCallSuper
  Future<void> onDidSave() async {}

  @protected
  @mustCallSuper
  void onCatchResponse(HttpsCallableResult<Map> response) {}

  @protected
  Map<String, dynamic> fromResponse(Map map) => map.cast<String, dynamic>();

  @protected
  @mustCallSuper
  Map<String, dynamic> filterOnCall(Map<String, dynamic> loaded) => loaded;

  Future<T> call({Map<String, dynamic>? parameters}) async {
    await FirebaseCore.initialize();
    await onLoad();
    final res = await functions
        .httpsCallable(endpoint.split("/").last)
        .call<Map>(parameters);
    onCatchResponse(res);
    value = fromMap(filterOnCall(fromResponse(res.data)));
    notifyListeners();
    await onDidLoad();
    return value;
  }

  @override
  // ignore: avoid_equals_and_hash_code_on_mutable_classes
  bool operator ==(Object other) => hashCode == other.hashCode;

  @override
  // ignore: avoid_equals_and_hash_code_on_mutable_classes
  int get hashCode => super.hashCode ^ endpoint.hashCode;
}
